/*
 * Jalview - A Sequence Alignment Editor and Viewer ($$Version-Rel$$)
 * Copyright (C) $$Year-Rel$$ The Jalview Authors
 * 
 * This file is part of Jalview.
 * 
 * Jalview is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3
 * of the License, or (at your option) any later version.
 *  
 * Jalview is distributed in the hope that it will be useful, but 
 * WITHOUT ANY WARRANTY; without even the implied warranty 
 * of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with Jalview.  If not, see <http://www.gnu.org/licenses/>.
 * The Jalview Authors are detailed in the 'AUTHORS' file.
 */
package jalview.ext.jmol;

import static org.testng.Assert.assertEquals;
import static org.testng.Assert.assertTrue;

import jalview.api.AlignmentViewPanel;
import jalview.api.FeatureRenderer;
import jalview.datamodel.Alignment;
import jalview.datamodel.AlignmentI;
import jalview.datamodel.ColumnSelection;
import jalview.datamodel.HiddenColumns;
import jalview.datamodel.Sequence;
import jalview.datamodel.SequenceI;
import jalview.ext.rbvi.chimera.AtomSpecModel;
import jalview.gui.AlignFrame;
import jalview.gui.JvOptionPane;
import jalview.gui.SequenceRenderer;
import jalview.schemes.ColourSchemeI;
import jalview.schemes.JalviewColourScheme;
import jalview.structure.AtomSpec;
import jalview.structure.StructureMapping;
import jalview.structure.StructureMappingcommandSet;
import jalview.structure.StructureSelectionManager;
import jalview.structures.models.AAStructureBindingModel;
import jalview.util.StructureCommands;

import java.awt.Color;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

import junit.extensions.PA;

public class JmolCommandsTest
{
  private SequenceRenderer sr;

  private String[] files;

  private AAStructureBindingModel mockBinding = new AAStructureBindingModel(
          null, null)
  {
    @Override
    public void releaseReferences(Object svl)
    {
    }

    @Override
    public void highlightAtoms(List<AtomSpec> atoms)
    {
    }

    @Override
    public void setJalviewColourScheme(ColourSchemeI cs)
    {
    }

    @Override
    public String superposeStructures(AlignmentI[] alignments,
            int[] structureIndices, HiddenColumns[] hiddenCols)
    {
      return null;
    }

    @Override
    public void setBackgroundColour(Color col)
    {
    }

    @Override
    public jalview.api.SequenceRenderer getSequenceRenderer(
            AlignmentViewPanel alignment)
    {
      return sr;
    }

    @Override
    protected void colourBySequence(AlignmentViewPanel avp)
    {
    }

    @Override
    public void colourByChain()
    {
    }

    @Override
    public void colourByCharge()
    {
    }

    @Override
    public FeatureRenderer getFeatureRenderer(AlignmentViewPanel alignment)
    {
      return null;
    }

    @Override
    public String[] getStructureFiles()
    {
      return files;
    }

    @Override
    public String getModelSpec(int model)
    {
      return "#" + String.valueOf(model);
    }
  };

  @BeforeClass(alwaysRun = true)
  public void setUpJvOptionPane()
  {
    JvOptionPane.setInteractiveMode(false);
    JvOptionPane.setMockResponse(JvOptionPane.CANCEL_OPTION);
  }

  /**
   * Test for the now deprecated version of getColourBySequenceCommand
   */
  @Test(groups = { "Functional" }, enabled = false)
  public void testGetColourBySequenceCommands_hiddenColumns()
  {
    /*
     * load these sequences, coloured by Strand propensity,
     * with columns 2-4 hidden
     */
    SequenceI seq1 = new Sequence("seq1", "MHRSQSSSGG");
    SequenceI seq2 = new Sequence("seq2", "MVRSNGGSSS");
    AlignmentI al = new Alignment(new SequenceI[] { seq1, seq2 });
    AlignFrame af = new AlignFrame(al, 800, 500);
    af.changeColour_actionPerformed(JalviewColourScheme.Strand.toString());
    ColumnSelection cs = new ColumnSelection();
    cs.addElement(2);
    cs.addElement(3);
    cs.addElement(4);
    af.getViewport().setColumnSelection(cs);
    af.hideSelColumns_actionPerformed(null);
    SequenceRenderer sr = new SequenceRenderer(af.getViewport());
    SequenceI[][] seqs = new SequenceI[][] { { seq1 }, { seq2 } };
    String[] files = new String[] { "seq1.pdb", "seq2.pdb" };
    StructureSelectionManager ssm = new StructureSelectionManager();

    /*
     * map residues 1-10 to residues 21-30 (atoms 105-150) in structures
     */
    HashMap<Integer, int[]> map = new HashMap<>();
    for (int pos = 1; pos <= seq1.getLength(); pos++)
    {
      map.put(pos, new int[] { 20 + pos, 5 * (20 + pos) });
    }
    StructureMapping sm1 = new StructureMapping(seq1, "seq1.pdb", "pdb1",
            "A", map, null);
    ssm.addStructureMapping(sm1);
    StructureMapping sm2 = new StructureMapping(seq2, "seq2.pdb", "pdb2",
            "B", map, null);
    ssm.addStructureMapping(sm2);

    StructureMappingcommandSet[] commands = JmolCommands
            .getColourBySequenceCommand(ssm, files, seqs, sr,
                    af.alignPanel);
    assertEquals(commands.length, 2);
    assertEquals(commands[0].commands.length, 1);

    String chainACommand = commands[0].commands[0];
    // M colour is #82827d == (130, 130, 125) (see strand.html help page)
    assertTrue(
            chainACommand.contains("select 21:A/1.1;color[130,130,125]")); // first one
    // H colour is #60609f == (96, 96, 159)
    assertTrue(chainACommand.contains(";select 22:A/1.1;color[96,96,159]"));
    // hidden columns are Gray (128, 128, 128)
    assertTrue(chainACommand
            .contains(";select 23-25:A/1.1;color[128,128,128]"));
    // S and G are both coloured #4949b6 == (73, 73, 182)
    assertTrue(
            chainACommand.contains(";select 26-30:A/1.1;color[73,73,182]"));

    String chainBCommand = commands[1].commands[0];
    // M colour is #82827d == (130, 130, 125)
    assertTrue(
            chainBCommand.contains("select 21:B/2.1;color[130,130,125]"));
    // V colour is #ffff00 == (255, 255, 0)
    assertTrue(chainBCommand.contains(";select 22:B/2.1;color[255,255,0]"));
    // hidden columns are Gray (128, 128, 128)
    assertTrue(chainBCommand
            .contains(";select 23-25:B/2.1;color[128,128,128]"));
    // S and G are both coloured #4949b6 == (73, 73, 182)
    assertTrue(
            chainBCommand.contains(";select 26-30:B/2.1;color[73,73,182]"));
  }

  @Test(groups = { "Functional" })
  public void testGetAtomSpec()
  {
    AtomSpecModel model = new AtomSpecModel();
    assertEquals(JmolCommands.getAtomSpec(model), "");

    /*
     * Jalview numbers models from 0, Jmol from 1
     */
    model.addRange(1, 2, 4, "A");
    assertEquals(JmolCommands.getAtomSpec(model), "2-4:A/2.1");

    model.addRange(1, 8, 8, "A");
    assertEquals(JmolCommands.getAtomSpec(model), "(2-4,8)&:A/2.1");

    model.addRange(1, 5, 7, "B");
    assertEquals(JmolCommands.getAtomSpec(model),
            "(2-4,8)&:A/2.1,5-7:B/2.1");

    model.addRange(1, 3, 5, "A");
    // 3-5 merges with 2-4 to make 2-5:
    assertEquals(JmolCommands.getAtomSpec(model),
            "(2-5,8)&:A/2.1,5-7:B/2.1");

    model.addRange(0, 1, 4, "B");
    assertEquals(JmolCommands.getAtomSpec(model),
            "1-4:B/1.1,(2-5,8)&:A/2.1,5-7:B/2.1");

    model.addRange(0, 5, 9, "C");
    assertEquals(JmolCommands.getAtomSpec(model),
            "1-4:B/1.1,5-9:C/1.1,(2-5,8)&:A/2.1,5-7:B/2.1");

    model.addRange(1, 8, 10, "B");
    // 8-10 extends 5-7 to make 5-10
    // note code generates (5-10)&:B which is equivalent to 5-10:B
    assertEquals(JmolCommands.getAtomSpec(model),
            "1-4:B/1.1,5-9:C/1.1,(2-5,8)&:A/2.1,(5-10)&:B/2.1");

    model.addRange(1, 8, 9, "B");
    // subsumed by 5-10 so makes no difference
    assertEquals(JmolCommands.getAtomSpec(model),
            "1-4:B/1.1,5-9:C/1.1,(2-5,8)&:A/2.1,(5-10)&:B/2.1");

    model.addRange(0, 3, 10, "C");
    // subsumes 5-9 so replaces it
    assertEquals(JmolCommands.getAtomSpec(model),
            "1-4:B/1.1,(3-10)&:C/1.1,(2-5,8)&:A/2.1,(5-10)&:B/2.1");

    // empty chain code - e.g. from homology modelling
    model.addRange(5, 25, 35, " ");
    assertEquals(JmolCommands.getAtomSpec(model),
            "1-4:B/1.1,(3-10)&:C/1.1,(2-5,8)&:A/2.1,(5-10)&:B/2.1,25-35:/6.1");
  }

  @Test(groups = "Functional")
  public void testGetColourBySequenceCommand()
  {
    /*
     * load these sequences, coloured by Strand propensity,
     * with columns 2-4 hidden (counting from 0)
     */
    SequenceI seq1 = new Sequence("seq1", "MHRSQSSSGG");
    SequenceI seq2 = new Sequence("seq2", "MVRSNGGMSS");
    AlignmentI al = new Alignment(new SequenceI[] { seq1, seq2 });
    AlignFrame af = new AlignFrame(al, 800, 500);
    af.changeColour_actionPerformed(JalviewColourScheme.Strand.toString());
    ColumnSelection cs = new ColumnSelection();
    cs.addElement(2);
    cs.addElement(3);
    cs.addElement(4);
    af.getViewport().setColumnSelection(cs);
    af.hideSelColumns_actionPerformed(null);
    sr = new SequenceRenderer(af.getViewport());
    SequenceI[][] seqs = new SequenceI[][] { { seq1 }, { seq2 } };
    files = new String[] { "seq1.pdb", "seq2.pdb" };
    StructureSelectionManager ssm = new StructureSelectionManager();

    /*
     * map residues 1-10 to residues 21-30 (atoms 105-150) in structures
     */
    HashMap<Integer, int[]> map = new HashMap<>();
    for (int pos = 1; pos <= seq1.getLength(); pos++)
    {
      map.put(pos, new int[] { 20 + pos, 5 * (20 + pos) });
    }
    StructureMapping sm1 = new StructureMapping(seq1, "seq1.pdb", "pdb1",
            "A", map, null);
    ssm.addStructureMapping(sm1);
    StructureMapping sm2 = new StructureMapping(seq2, "seq2.pdb", "pdb2",
            "B", map, null);
    ssm.addStructureMapping(sm2);

    /*
     * put data into the mock binding object
     */
    PA.setValue(mockBinding, "ssm", ssm);
    PA.setValue(mockBinding, "sequence", seqs);

    Map<Object, AtomSpecModel> colourMap = StructureCommands
            .buildColoursMap(mockBinding, af.alignPanel);

    String[] commands = JmolCommands.getColourBySequenceCommand(colourMap);

    assertEquals(commands.length, 5);

    /*
     * verify selections and colours match Strand Propensity colour scheme
     * (see strand.html help page for colours)
     *
     * M is #82827d == (130, 130, 125) at model 1 pos 21A, model 2 21B
     */
    assertEquals(commands[0],
            "color (21:A/1.1,(21,28)&:B/2.1) [130,130,125];");

    /*
     * H is #60609f == (96, 96, 159) 
     */
    assertEquals(commands[1], "color (22:A/1.1) [96,96,159];");

    /*
     * hidden columns are Gray (128, 128, 128)
     */
    assertEquals(commands[2],
            "color (23-25:A/1.1,23-25:B/2.1) [128,128,128];");

    /*
     * S and G are both coloured #4949b6 == (73, 73, 182)
     */
    assertEquals(commands[3],
            "color (26-30:A/1.1,(26-27,29-30)&:B/2.1) [73,73,182];");

    /*
     * V is ffff00 == (255, 255, 0)
     */
    assertEquals(commands[4], "color (22:B/2.1) [255,255,0];");
  }
}
